#GetPaid imports
from Products.PloneGetPaid.interfaces import IGetPaidManagementOptions
from getpaid.core import interfaces, options
from getpaid.core.order import Order
from getpaid.formgen.interfaces import ICreateTransientOrder #It is a getpaid import, with a tweak

#Local imports
from getpaid.formgen.interfaces import IMakePaymentProcess

#Zope imports
from zope import interface, component
from zope.app import zapi
from zope.event import notify
from zope.app.event.objectevent import ObjectCreatedEvent

#Plone imports
from Products.CMFCore.utils import getToolByName

#Other imports
from cPickle import loads, dumps
from AccessControl import getSecurityManager

class CreateTransientOrder( object ):
    """
    A really transient order
    """
    interface.implements(ICreateTransientOrder)

    def __call__( self, adapters, shopping_cart=None ):
        order = Order()

        portal = zapi.getSiteManager()
        portal = getToolByName(portal,'portal_url').getPortalObject()
        if shopping_cart is None:
            """
            This deservers a bit of explanation, when using the formgen adapter
            for 'disposable' carts (this means that we don't use the site's
            cart but a transient one just for the ocasion) we want this cart
            to be different from the user's one and to cease to exist after
            the transaction (means no persistence) .
            """
            shopping_cart =  component.getUtility( interfaces.IShoppingCartUtility ).get( portal )
        formSchemas = component.getUtility( interfaces.IFormSchemas )

        order.shopping_cart = shopping_cart

        for section in ('contact_information','billing_address','shipping_address'):
            interface = formSchemas.getInterface(section)
            bag = formSchemas.getBagClass(section).frominstance(adapters[interface])
            setattr(order,section,bag)

        order_manager = component.getUtility( interfaces.IOrderManager )        
        order.order_id = order_manager.newOrderId()
        
        order.user_id = getSecurityManager().getUser().getId()

        return order

class MakePaymentProcess( object ):
    """
    The generic steps included in the make payment steps
    """
    interface.implements(IMakePaymentProcess)
        
    def __init__( self, context, adapters ):
        manage_options = IGetPaidManagementOptions( context )
        processor_name = manage_options.payment_processor
        if not processor_name:
            raise RuntimeError( "No Payment Processor Specified" )
        self.processor_name = processor_name
        self.processor = component.getAdapter( context,
                                               interfaces.IPaymentProcessor,
                                               processor_name )
        self.adapters = adapters
        #self.order = CreateTransientOrder()
        self.order = None
        self.context = context
        
    def __call__( self, oneshot=None ):
        """
        If called as oneshot it will not use the site's cart, instead oneshot
        should be the cart to use
        """

        shopping_cart = oneshot
        if oneshot is None:
            #If this is not to be a disposable cart transaction we get the cart as we should.
            #NOTICE: This has not been tested since there is no process currently that uses it
            shopping_cart = component.getUtility( interfaces.IShoppingCartUtility ).get( self.context )
            shopping_cart = loads( dumps( shopping_cart ) )

        order_factory = CreateTransientOrder()
        self.order = order_factory( self.adapters, shopping_cart )
        notify( ObjectCreatedEvent( self.order ) )
        self.order.processor_id = self.processor_name
        self.order.finance_workflow.fireTransition( "create" )
        
        # extract data to our adapters
        formSchemas = component.getUtility(interfaces.IFormSchemas)
        result = self.processor.authorize( self.order, self.adapters[formSchemas.getInterface('payment')] )
        if result is interfaces.keys.results_async:
            # shouldn't ever happen, on async processors we're already directed to the third party
            # site on the final checkout step, all interaction with an async processor are based on processor
            # adapter specific callback views.
            pass
        elif result is interfaces.keys.results_success:
            order_manager = component.getUtility( interfaces.IOrderManager )
            order_manager.store( self.order )
            self.order.finance_workflow.fireTransition("authorize")
            # kill the cart after we create the order
            component.getUtility( interfaces.IShoppingCartUtility ).destroy( self.context )
            return None
        else:
            self.order.finance_workflow.fireTransition('reviewing-declined')
            return  result




"""This is a a functional doctest test. It uses PloneTestCase and doctest
syntax. In the test itself, we use zope.testbrowser to test end-to-end
functionality, including the UI.

One important thing to note: zope.testbrowser is not JavaScript aware! For
that, you need a real browser. Look at zope.testbrowser.real and Selenium
if you require "real" browser testing.
"""

import os, sys

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

import unittest
import doctest
from Testing.ZopeTestCase import FunctionalDocFileSuite as Suite
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import _createObjectByType
from getpaid.formgen.tests import base_test_case
from getpaid.formgen.config import PROJECTNAME

from Products.PloneTestCase.PloneTestCase import PloneTestCase
from Products.PloneTestCase.PloneTestCase import setupPloneSite
from Products.PloneTestCase.PloneTestCase import FunctionalTestCase

setupPloneSite(products=base_test_case.PRODUCTS)

from getpaid.formgen.tests.util import list_acceptance_doctests

class AcceptanceFunctionalTest(FunctionalTestCase):
    def afterSetUp(self):
        pass

def test_suite():
    """This sets up a test suite that actually runs the tests in the class
    above
    """
    suite = unittest.TestSuite()
    filenames = list_acceptance_doctests()

    for docfile in filenames:     
        suite.addTest(Suite(docfile,
                            package='getpaid.formgen.tests',
                            test_class=AcceptanceFunctionalTest))

    return suite



# Adapted from SalesforcePFGAdapter source code.

from getpaid.formgen.config import PROJECTNAME
from getpaid.formgen import HAS_PLONE30

from Products.CMFCore.utils import getToolByName
from Products.CMFCore.permissions import ManagePortal
from Products.CMFPlone.utils import versionTupleFromString

from StringIO import StringIO

ALLTYPES = ('GetpaidPFGAdapter','GetPaidFormMailerAdapter',)
DEPENDENCIES = ('PloneFormGen','PloneGetPaid',) # eventually DataGridField as well

def install(self):
    out = StringIO()
    
    print >> out, u"Installing dependency products"
    portal_qi = getToolByName(self, 'portal_quickinstaller')
    for depend in DEPENDENCIES:
        if portal_qi.isProductInstallable(depend) and not portal_qi.isProductInstalled(depend):
            portal_qi.installProduct(depend)
    
    print >> out, u"Installing GetpaidPFGAdapter"
    setup_tool = getToolByName(self, 'portal_setup')
    if HAS_PLONE30:
        setup_tool.runAllImportStepsFromProfile(
                "profile-getpaid.formgen:default",
                purge_old=False)
    else:
        pass #raise ValueError(u"Plone >= 3.0 required.")
    print >> out, "Installed types and added to portal_factory via portal_setup"
    
    
    # add the GetpaidPFGAdapter type as an addable type to FormField
    types_tool = getToolByName(self, 'portal_types')
    if 'FormFolder' in types_tool.objectIds():
        allowedTypes = types_tool.FormFolder.allowed_content_types
        
        for f in ALLTYPES:
            if f not in allowedTypes:
                allowedTypes = list(allowedTypes)
                allowedTypes.append(f)
                types_tool.FormFolder.allowed_content_types = allowedTypes
                print >> out, u"Added %s to Form Field allowed_content_types" % f
        
    propsTool = getToolByName(self, 'portal_properties')
    siteProperties = getattr(propsTool, 'site_properties')
    navtreeProperties = getattr(propsTool, 'navtree_properties')

    # Add the field, fieldset, thanks and adapter types to types_not_searched
    # This is not desirable to do with GS because we don't want to maintain a list of 
    # the Portal's types_not_searched and we don't want to overwrite existing settings
    typesNotSearched = list(siteProperties.getProperty('types_not_searched'))
    for f in ALLTYPES:
        if f not in typesNotSearched:
            typesNotSearched.append(f)
    siteProperties.manage_changeProperties(types_not_searched = typesNotSearched)
    print >> out, "Added form fields & adapters to types_not_searched"
    
    # Add the field, fieldset, thanks and adapter types to types excluded from navigation
    # This is not desirable to do with GS because we don't want to maintain a list of 
    # the Portal's metaTypesNotToList and we don't want to overwrite existing settings
    typesNotListed = list(navtreeProperties.getProperty('metaTypesNotToList'))
    for f in ALLTYPES:
        if f not in typesNotListed:
            typesNotListed.append(f)
    navtreeProperties.manage_changeProperties(metaTypesNotToList = typesNotListed)
    print >> out, "Added form fields & adapters to metaTypesNotToList"
    
    print >> out, "Successfully installed %s." % PROJECTNAME
    return out.getvalue()


def uninstall(self):
    out = StringIO()
    
    portal_factory = getToolByName(self,'portal_factory')
    propsTool = getToolByName(self, 'portal_properties')
    siteProperties = getattr(propsTool, 'site_properties')
    navtreeProperties = getattr(propsTool, 'navtree_properties')
    types_tool = getToolByName(self, 'portal_types')
    
    # remove salesforce adapter as a factory type
    factory_types = portal_factory.getFactoryTypes().keys()
    for t in ALLTYPES:
        if t in factory_types:
            factory_types.remove(t)
    portal_factory.manage_setPortalFactoryTypes(listOfTypeIds=factory_types)
    print >> out, "Removed form adapters from portal_factory tool"
    
    # remove salesforce adapter from Form Folder's list of addable types
    if 'FormFolder' in types_tool.objectIds():
        allowedTypes = types_tool.FormFolder.allowed_content_types
        
        for t in ALLTYPES:
            if t in allowedTypes:
                allowedTypes = list(allowedTypes)
                allowedTypes.remove(t)
                types_tool.FormFolder.allowed_content_types = allowedTypes
                print >> out, u"Removed %s from FormFolder's allowedTypes" % t
    
    # remove our types from the portal's list of types excluded from navigation
    typesNotListed = list(navtreeProperties.getProperty('metaTypesNotToList'))
    for f in ALLTYPES:
        if f in typesNotListed:
            typesNotListed.remove(f)
    navtreeProperties.manage_changeProperties(metaTypesNotToList = typesNotListed)
    print >> out, "Removed form adapters from metaTypesNotToList"

    # remove our types from the portal's list of types_not_searched
    typesNotSearched = list(siteProperties.getProperty('types_not_searched'))
    for f in ALLTYPES:
        if f in typesNotSearched:
            typesNotSearched.remove(f)
    siteProperties.manage_changeProperties(types_not_searched = typesNotSearched)
    print >> out, "Removed form adapters from types_not_searched"
    
    # # Remove skin directory from skin selections
    # skinstool = getToolByName(self, 'portal_skins')
    # for skinName in skinstool.getSkinSelections():
    #     path = skinstool.getSkinPath(skinName)
    #     path = [i.strip() for i in  path.split(',')]
    #     if 'salesforcepfgadapter_images' in path:
    #         path.remove('salesforcepfgadapter_images')
    #         path = ','.join(path)
    #         skinstool.addSkinSelection(skinName, path)
    # print >> out, "Removed salesforcepfgadapter_images layer from all skin selections"
    
    print >> out, "\nSuccessfully uninstalled %s." % PROJECTNAME
    return out.getvalue()




from zope import schema
from zope.interface import Interface

from zope.app.container.constraints import contains
from zope.app.container.constraints import containers

from getpaid.formgen import GPFGMessageFactory as _

class IMakePaymentProcess(Interface):
    """
    Fulfillment geric steps
    """
try:
    from Products.PloneGetPaid.interfaces import ICreateTransientOrder
except:
    #This is only to make it work with current version of getpaid (0.6.1)
    class ICreateTransientOrder(Interface):
        """
        A transient order used by checkout forms and finally persisted
        """





# -*- encoding: utf-8 -*-

from setuptools import setup, find_packages

version = '0.2.4'

setup(name='getpaid.formgen',
      version=version,
      description=u"PloneGetPaid-PloneFormGen integration".encode('utf-8'),
      long_description=open("README.txt").read() + "\n" +
                       open("CHANGES.txt").read(),
      classifiers=[
        "Framework :: Plone",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "License :: OSI Approved :: GNU General Public License (GPL)"
        ],
      keywords='plone plonegetpaid ploneformgen',
      author='Daniel Holth',
      author_email='daniel.holth@exac.com',
      url='http://dingoskidneys.com/cgi-bin/hgwebdir.cgi/getpaid.formgen',
      packages=find_packages('src'),
      package_dir={'':'src'},
      namespace_packages=['getpaid'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'Products.DataGridField >= 1.6rc1',
          'Products.PloneFormGen'
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



import os, sys
import glob

from Globals import package_home
from getpaid.formgen.tests import product_globals

def list_acceptance_doctests():
    home = package_home(product_globals) + "/tests/"
    return [os.path.basename(filename) for filename in
            glob.glob(os.path.sep.join([home, 'test*.txt']))]

